<?php
/**
 * API du plugin Contours.
 *
 * @package SPIP\CONTOURS\API
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * Ajoute des contours géolocalisés aux territoires.
 * Les territoires sont identifiés par groupe via le couple (type, pays).
 *
 * @api
 *
 * @param string $type    Type de territoires à peupler. Prends les valeurs `zone`, `country` ou `subdivision`.
 * @param string $pays    Code ISO 3166-1 alpha2 du pays dont on veut peupler les subdivisions.
 *                        N'est utilisé que si le type choisi est `subdivision`.
 * @param string $service Identifiant du service Nomenclatures ayant permis le chargement. Est utilisé pour indexer
 *                        la consignation dans le cas des contours uniquement. Sinon vaut chaine vide
 *
 * @return array Tableau retour de la fonction permettant de connaitre le résultat du traitement (utile pour l'affichage
  *               du message dans le formulaire de peuplement).
 */
function contour_peupler(string $type, string $pays, string $service) : array {
	// On initialise le retour à une erreur nok
	$retour = [
		'ok'      => false,
		'arg'     => false,
		'sha'     => false,
		'type'    => $type,
		'pays'    => $pays,
		'service' => $service
	];

	if (
		include_spip('inc/territoires_services')
		and type_pays_est_valide($type, $pays)
	) {
		$erreur = true;
		$type_identique = false;

		// Identification de la meta et de la variable de consigne
		include_spip('inc/contours_utils');
		$meta = 'contours_peuplement';
		$consigne = consigne_contour_identifier($type, $pays, $service);

		// Détermination du service et acquisition des contours
		$collection = contours_acquerir($type, $pays, $service);
		if (!empty($collection['contours'])) {
			// On extrait que l'index correspondant aux contours
			$contours = $collection['contours'];

			// Si les contours sont déjà chargés le sha a été consigné. Si le sha des contours récupérées
			// possèdent le même sha alors on ne fait aucun traitement et on indique que la configuration n'a pas
			// changée. Dans ce cas, aucun traitement n'a lieu.
			$sha_type = sha1(json_encode($contours));
			if (sha_est_identique($sha_type, $meta, $consigne)) {
				$type_identique = true;
			} else {
				// On vide les contours, cad les objets GIS et les liens gis-territoire, avant de les remettre
				// (inutile de gérer les erreurs car l'insertion les détectera).
				contour_depeupler($type, $pays, $service);

				// On extrait la liste des codes alternatifs si elle est fournie par l'API.
				// (la liste n'est fournie que si les contours sont liés à un code qui n'est pas celui de base des
				// territoires).
				$codes = [];
				if (!empty($collection['codes_alternatifs'])) {
					$codes = array_column($collection['codes_alternatifs'], 'code_iso', 'code_alter');
				}

				// On insère chaque contour comme un objet GIS et on crée son lien avec le territoire rattaché
				include_spip('action/editer_objet');
				include_spip('action/editer_gis');
				$erreur_insertion = false;
				$nb_contours = 0;
				foreach ($contours as $_contour) {
					// Initialisation de l'enregistrement d'un objet GIS représentant le contour
					$enregistrement = [];

					// Détermination du territoire : si le territoire n'existe pas on n'insère pas le contour
					$id_territoire = 0;
					if ($_contour['code_type'] !== 'code_iso') {
						// La collection est fournie avec la tableau de conversion vers le code iso du territoire
						$code_iso = isset($codes[$_contour['code']]) ? $codes[$_contour['code']] : '';
					} else {
						$code_iso = $_contour['code'];
					}
					if ($code_iso) {
						$options = [
							'champs'   => 'id_territoire',
							'champ_id' => 'iso_territoire'
						];
						$id_territoire = objet_lire('territoire', $code_iso, $options);
					}

					// -- Champs recopiés dans l'objet GIS
					if ($id_territoire) {
						$enregistrement['lat'] = $_contour['lat'];
						$enregistrement['lon'] = $_contour['lon'];
						// -- Champs descriptifs : le titre est composé du service et de l'identifiant pour le service
						$enregistrement['titre'] = "{$service}-{$_contour['code']}";
						$enregistrement['descriptif'] = '';
						// -- Champs de la géométrie : on insère en premier lieu le point central (lat, lon) pour
						//    que la requête fonctionne car le champ geo ne supporte pas d'être vide
						$centre = [
							'type'        => 'Point',
							'coordinates' => [
								(int) ($enregistrement['lon']),
								(int) ($enregistrement['lat'])
							]
						];
						$json = json_encode($centre);
						$geometrie = sql_getfetsel("ST_GeomFromGeoJSON('{$json}')");
						$enregistrement['geo'] = $geometrie;

						// Insertion de l'objet GIS
						if ($id = sql_insertq('spip_gis', $enregistrement)) {
							// Maintenant que l'objet GIS est créé on met à jour sa géométrie avec le contour véritable
							$json = json_encode($_contour['geometry']);
							$geo = ['geo' => "ST_GeomFromGeoJSON('{$json}')"];
							sql_update('spip_gis', $geo, 'id_gis=' . sql_quote($id));

							// Ajout du lien entre le point GIS et le territoire
							gis_associer($id, ['territoire' => $id_territoire]);
							++$nb_contours;
						} else {
							$erreur_insertion = true;
							break;
						}
					} else {
						spip_log("Le territoire (Type '{$type}' - Pays '{$pays}' - Code '{$_contour['code']}') n'a pas de code ISO correspondant", 'contours' . _LOG_ERREUR);
					}
				}

				if (!$erreur_insertion) {
					// On stocke les informations de chargement dans une meta.
					$contenu = [
						'sha' => $sha_type,
						'nbr' => count($contours),
						'maj' => date('Y-m-d H:i:s'),
						'lic' => isset($collection['credits']) ? $collection['credits'] : [],
					];
					include_spip('inc/config');
					ecrire_config("{$meta}/{$consigne}", $contenu);
					$erreur = false;
					spip_log("Les '{$nb_contours}' contours (Type '{$type}' - Pays '{$pays}' - Service  '{$service}') ont été chargés", 'contours' . _LOG_DEBUG);
				}
			}
		} else {
			spip_log("Aucun contour pour (Type '{$type}' - Pays '{$pays}' - Service  '{$service}') retourné par Nomenclatures", 'contours' . _LOG_ERREUR);
		}

		// Si le territoire est en erreur, on passe on stocke le cas d'erreur.
		if ($erreur) {
			if ($type_identique) {
				$retour['sha'] = true;
				spip_log("Les contours de (Type '{$type}' - Pays '{$pays}' - Service  '{$service}') sont inchangés", 'territoires' . _LOG_AVERTISSEMENT);
			} else {
				$retour['ok'] = false;
			}
		} else {
			$retour['ok'] = true;
		}
	} else {
		$retour['arg'] = true;
		spip_log("Le couple (Type '{$type}' - Pays '{$pays}') est invalide", 'contours' . _LOG_ERREUR);
	}

	return $retour;
}

/**
 * Supprime des contours géolocalisés de territoires.
 * Les territoires sont identifiés par groupe via le couple (type, pays).
 *
 * @api
 *
 * @param string $type    Type de territoires à peupler. Prends les valeurs `zone`, `country` ou `subdivision`.
 * @param string $pays    Code ISO 3166-1 alpha2 du pays dont on veut peupler les subdivisions.
 *                        N'est utilisé que si le type choisi est `subdivision`, sinon vaut chaine vide.
 * @param string $service Identifiant du service Nomenclatures ayant permis le chargement. Est utilisé pour indexer
 *                        la consignation dans le cas des contours uniquement. Sinon vaut chaine vide
 *
 * @return array Liste des code ISO 3166-1 alpha2 des pays chargés sous la forme [code] = nom multi.
 */
function contour_depeupler($type, $pays, $service) {
	// On initialise le retour à une erreur nok
	$retour = [
		'ok'      => false,
		'arg'     => false,
		'sha'     => false,
		'type'    => $type,
		'pays'    => $pays,
		'service' => $service
	];

	if (
		include_spip('inc/territoires_services')
		and type_pays_est_valide($type, $pays)
	) {
		// Inutile de vider une table vide
		if (contour_est_peuple($type, $pays, $service)) {
			// Identification de la variable de consigne
			include_spip('inc/contours_utils');
			$consigne = consigne_contour_identifier($type, $pays, $service);

			// Avant de vider la table on réserve la liste des id de territoire qui seront supprimés
			// de façon à vider ensuite les liens éventuels y compris ceux des logos.
			// -- on calcule donc d'emblée la condition IN qui sera appliquée sur la table de liens.
			$from = 'spip_gis';
			$where = [
				'titre LIKE ' . sql_quote("{$service}-%"),
			];
			$where_lien[] = sql_in_select('id_gis', 'id_gis', $from, $where);

			$sql_ok = sql_delete($from, $where);
			if ($sql_ok !== false) {
				// Vider les liens gis_liens
				$where_lien[] = 'objet=' . sql_quote('territoire');
				sql_delete('spip_gis_liens', $where_lien);

				// Supprimer la meta propre au service de contours.
				include_spip('inc/config');
				effacer_config("contours_peuplement/{$consigne}");
			}

			// Enregistrer le succès ou pas du déchargement de la table
			if ($sql_ok !== false) {
				// Succès complet
				$retour['ok'] = true;
				spip_log("Les contours (Type '{$type}' - Pays '{$pays}' - Service  '{$service}') ont été vidés avec succès", 'contours' . _LOG_DEBUG);
			} else {
				spip_log("Erreur de vidage des contours (Type '{$type}' - Pays '{$pays}' - Service  '{$service}')", 'contours' . _LOG_ERREUR);
			}
		} else {
			$retour['sha'] = true;
			spip_log("Aucun contour (Type '{$type}' - Pays '{$pays}' - Service  '{$service}') à vider", 'contours' . _LOG_AVERTISSEMENT);
		}
	} else {
		$retour['arg'] = true;
		spip_log("Le couple (Type '{$type}' - Pays '{$pays}') est invalide", 'contours' . _LOG_ERREUR);
	}

	return $retour;
}

/**
 * Renvoie la liste des pays dont les territoires ont été chargées.
 * La fonction lit la meta de chargement et non la table spip_territoires.
 *
 * @api
 *
 * @param string $type    Type de territoires à acquérir. Prends les valeurs `zone`, `country` ou `subdivision`.
 * @param string $pays    Code ISO 3166-1 alpha2 du pays si le type est une subdivision ou chaine vide sinon.
 * @param string $service Identifiant du service.
 *
 * @return bool true si le territoire est chargé, false sinon.
 */
function contour_est_peuple(string $type, string $pays, string $service) : bool {
	// Définir la variable de config
	include_spip('inc/contours_utils');
	$consigne = consigne_contour_identifier($type, $pays, $service);

	// La liste des territoires chargés est en meta.
	include_spip('inc/config');

	return lire_config("contours_peuplement/{$consigne}", [])
		? true
		: false;
}
